/* -------------------------------------------------------------------------- *
 * file:         verifyzengin.c                                               *
 * purpose:      shows the content from buildzengin.c and writes it to file   *
 * ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <cgic.h>
#include "zengingen.h"

void pad_zero(char var[], int max) {
  char  tmp[255]   = "";
  int   j          =0;
  int len          =0;

  len = strlen(var);
  if (len<max) {
    strncpy(tmp, var, len);
    for(j=0;j<max-len;j++) { var[j] = '0'; }
    var[j+1] = '\0';
    strncat(var, tmp, len);
  }
}

void pad_space(char var[], int max) {
  int   j          =0;
  int len          =0;

  len = strlen(var);
  if (len<max) {
    for(j=len;j<max;j++) var[j] = ' ';
    var[j+1] = '\0';
  }
}

int cgiMain() {
  int i = 0;
  static char title[]          = "Verify the Zengin File";
  const char crlf[3]           = {'\r', '\n', '\0'};
  static int tlr_field_length[TRAILER_FIELDS] = {1, 6, 12, 101};

  const char end_f1[1+1] = "9";
  char end_f2[119+1];

  strncpy(hdr[0], "1", 2);
  strncpy(data[0], "2", 2);
  strncpy(hdr[13], "              ", 15);
  strncpy(data[15], "       ", 8);
  strncpy(tlr[0], "8", 2);
  strncpy(tlr[1], "000001", 7);

  if (cgiFormCheckboxSingle("crlf") == cgiFormSuccess)
    zengin_fmt = CRLF_FMT;

/* ------------------------------------------------------------------------- *
 * sanitize and pad the header fields                                        *
 * --------------------------------------------------------------------------*/
  cgiFormString("hdr_f2", hdr[1], 2+1);
  cgiFormString("hdr_f3", hdr[2], 1+1);
  cgiFormString("hdr_f4", hdr[3], 10+1);   // leading 0 padding to 10
  pad_zero(hdr[3], 10);

  cgiFormString("hdr_f5", hdr[4], 40+1);   // trailing space padding to 40
  pad_space(hdr[4], 40);

  cgiFormString("hdr_f6", hdr[5], 4+1);
  cgiFormString("hdr_f7", hdr[6], 4+1);
  cgiFormString("hdr_f8", hdr[7], 15+1);    // trailing space padding to 15
  pad_space(hdr[7], 15);

  cgiFormString("hdr_f9", hdr[8], 3+1);
  cgiFormString("hdr_f10", hdr[9], 15+1);   // trailing space padding to 15
  pad_space(hdr[9], 15);

  cgiFormString("hdr_f11", hdr[10], 1+1);
  cgiFormString("hdr_f12", hdr[11], 7+1); //leading 0 padding to 7
  pad_zero(hdr[11], 7);

  cgiFormString("hdr_f13", hdr[12], 3+1); // if empty pad with spaces
  if(strcmp(hdr[12], "JPY") == 0) strcpy(hdr[12], "   ");

/* ------------------------------------------------------------------------- *
 * sanitize and pad the data fields                                          *
 * --------------------------------------------------------------------------*/
  cgiFormString("data_f2", data[1], 4+1);
  cgiFormString("data_f3", data[2], 15+1); // trailing space padding to 15
  pad_space(data[2], 15);

  cgiFormString("data_f4", data[3], 3+1);
  cgiFormString("data_f5", data[4], 15+1); // trailing space padding to 15
  pad_space(data[4], 15);

  cgiFormString("data_f6", data[5], 4+1);
  cgiFormString("data_f7", data[6], 1+1);
  cgiFormString("data_f8", data[7], 7+1); //leading 0 padding to 7
  pad_zero(data[7], 7);

  cgiFormString("data_f9", data[8], 30+1); // trailing space padding to 30
  pad_space(data[8], 30);

  cgiFormString("data_f10", data[9], 10+1); //leading 0 padding to 10
  pad_zero(data[9], 10);

  cgiFormString("data_f11", data[10], 1+1); 
  cgiFormString("data_f12", data[11], 10+1); // trailing space padding to 10
  pad_space(data[11], 10);

  cgiFormString("data_f13", data[12], 10+1); // trailing space padding to 10
  pad_space(data[12], 10);

  cgiFormString("data_f14", data[13], 1+1);
  cgiFormString("data_f15", data[14], 1+1);

  cgiFormString("tlr_f3", tlr[2],  tlr_field_length[2]+1);
  pad_zero(tlr[2], 12);

/* -------------------------------------------------------------------------- *
 * write the header record to file                                            *
 * ---------------------------------------------------------------------------*/
  for(i=0;i<HDR_FIELDS;i++) {
    if(strlen(hdr[i]) == 0) {
      snprintf(errorstr,sizeof(errorstr), "Header Field %d is empty!", i);
      int_error(errorstr);
    }
    strncat(zengin_filestr, hdr[i], sizeof(hdr[i]));
  }
  if (zengin_fmt == CRLF_FMT) strncat(zengin_filestr, crlf, sizeof(crlf));
/* -------------------------------------------------------------------------- *
 * write the data record to file                                              *
 * ---------------------------------------------------------------------------*/
  for(i=0;i<DATA_FIELDS;i++) {
    if(strlen(data[i]) == 0) int_error("Data Field is empty!");
    strncat(zengin_filestr, data[i], sizeof(data[i]));
  }
  if (zengin_fmt == CRLF_FMT) strncat(zengin_filestr, crlf, sizeof(crlf));
/* -------------------------------------------------------------------------- *
 * write the trailer record to file                                           *
 * ---------------------------------------------------------------------------*/
  for(i=0;i<101;i++) { tlr[3][i] = ' '; }
  tlr[3][101] = '\0';

  for(i=0;i<TRAILER_FIELDS;i++) {
    if(strlen(tlr[i]) == 0) int_error("Trailer Field is empty!");
    strncat(zengin_filestr, tlr[i], sizeof(tlr[i]));
  }
  if (zengin_fmt == CRLF_FMT) strncat(zengin_filestr, crlf, sizeof(crlf));
/* -------------------------------------------------------------------------- *
 * write the end record to file                                               *
 * ---------------------------------------------------------------------------*/
  for(i=0;i<119;i++) { end_f2[i] = ' '; }
  end_f2[119] = '\0';
  strncat(zengin_filestr, end_f1, sizeof(end_f1));
  strncat(zengin_filestr, end_f2, sizeof(end_f2));
  if (zengin_fmt == CRLF_FMT) strncat(zengin_filestr, crlf, sizeof(crlf));
/* -------------------------------------------------------------------------- *
 * read the zengin file content                                               *
 * ---------------------------------------------------------------------------*/
  zengin_read(zengin_filestr);
  b64_encode(zengin_filestr, zengin_b64str);
/* -------------------------------------------------------------------------- *
 * start the html output                                                      *
 * ---------------------------------------------------------------------------*/
   pagehead(title);
  fprintf(cgiOut, "<p><center>");
  write_rawtable(0);
/* -------------------------------------------------------------------------- *
 * start the form output                                                      *
 * ---------------------------------------------------------------------------*/
  fprintf(cgiOut, "<p>");
  fprintf(cgiOut, "<table border=\"1\" cellspacing=\"0\" ");
  fprintf(cgiOut, "cellpadding=\"2\">");
  fprintf(cgiOut, "<tr>\n");

  fprintf(cgiOut, "<form action=\"buildzengin.cgi\" method=\"post\" ");
  fprintf(cgiOut, "enctype=\"multipart/form-data\">");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"zengindata\" value=%s>", zengin_b64str);
  fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\">");
  fprintf(cgiOut, " <input type=\"submit\" value=\"Back to Previous Page\">");
  fprintf(cgiOut, "</td>");
  fprintf(cgiOut, "</form>\n");

  /* udate existing file if we got zfilename */
  if (cgiFormString("zfilename", zfilename, sizeof(zfilename)) == cgiFormSuccess) {
    fprintf(cgiOut, "<form action=\"writezengin.cgi\" method=\"post\" ");
    fprintf(cgiOut, "enctype=\"multipart/form-data\">");
    fprintf(cgiOut, "<input type=\"hidden\" name=\"zfilename\" value=%s>", zfilename);
    fprintf(cgiOut, "<input type=\"hidden\" name=\"zengindata\" value=%s>", zengin_b64str);
    fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\">");
    fprintf(cgiOut, " <input type=\"submit\" value=\"Update The Current File\">");
    fprintf(cgiOut, "</td>");
    fprintf(cgiOut, "</form>\n");
  }

  /* create a new file */
  fprintf(cgiOut, "<form action=\"writezengin.cgi\" method=\"post\" ");
  fprintf(cgiOut, "enctype=\"multipart/form-data\">");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"zengindata\" value=%s>", zengin_b64str);
  fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\">");
  fprintf(cgiOut, " <input type=\"submit\" value=\"Write Into A New File\">");
  fprintf(cgiOut, "</td>");
  fprintf(cgiOut, "</form>\n");

  pagefoot();
  return(0);
}
