/* -------------------------------------------------------------------------- *
 * file:         writezengin.c                                                *
 * purpose:      writes zengin data from verifyzengin or decodezengin to file *
 * ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <cgic.h>
#include "zengingen.h"

int cgiMain() {
 struct tm writetime;
 time_t ts;
 char writetimestr[16];
 int bytecount = 0;
 char formbyte[10];
 char byte[3];

  /* Check if we got the file data or single bytes uploaded */
  if(cgiFormString("zengindata", zengin_b64str, sizeof(zengin_b64str)) == cgiFormSuccess)
    b64_decode(zengin_b64str, zengin_filestr);
  else {
    if(cgiFormString("zen_c0", zengin_filestr, sizeof(zengin_filestr)) == cgiFormSuccess) {
      while(1) {
        snprintf(formbyte, sizeof(formbyte), "zen_c%d", bytecount);
        if(cgiFormString(formbyte, byte, sizeof(byte)) != cgiFormSuccess) break;
        zengin_filestr[bytecount] = byte[0];
        bytecount++;
      }
      zengin_filestr[bytecount+1] = '\0';
      add_crlf();
    } else int_error("Error getting Zengin File Data for Writing.");
  }

  /* Check if we update an existing file or generate a new one */
  if(cgiFormString("zfilename", zfilename, sizeof(zfilename)) == cgiFormSuccess) {
    snprintf(title, sizeof(title), "%s %s", "Updating Zengin File", zfilename);
  } else {
    /* get the time */
    ts = time(NULL);
    writetime = *localtime(&ts);
    /* convert the timestamp into a human readable string */
    strftime(writetimestr, sizeof(writetimestr), "%Y%m%d-%H%M%S", &writetime);
    snprintf(zfilename, sizeof(zfilename), "%s-%s", "zengin", writetimestr);
    snprintf(title, sizeof(title), "%s %s", "Writing New Zengin File", zfilename);
  }
/* -------------------------------------------------------------------------- *
 * before we write, read the zengin file content to avoid writing garbage     *
 * ---------------------------------------------------------------------------*/
  zengin_read(zengin_filestr);

/* -------------------------------------------------------------------------- *
 * check if we can write the requested filename                               *
 * ---------------------------------------------------------------------------*/
  snprintf(zfilepath, sizeof(zfilepath), "%s/%s", ZENGINSTORE, zfilename);
  if (! (zenginfile = fopen(zfilepath, "w"))) int_error("Error cant open zengin file");
  fputs(zengin_filestr, zenginfile);
  fclose(zenginfile);

/* -------------------------------------------------------------------------- *
 * start the html output                                                      *
 * ---------------------------------------------------------------------------*/
   pagehead(title);
/* -------------------------------------------------------------------------- *
 * start the form output                                                      *
 * ---------------------------------------------------------------------------*/
  fprintf(cgiOut, "<p><center>");
  write_rawtable(0);

  fprintf(cgiOut, "<p><center>");
  fprintf(cgiOut, "<table border=\"1\" cellspacing=\"0\" ");
  fprintf(cgiOut, "cellpadding=\"2\">");
  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<form action=\"showzengin.cgi\" method=\"post\">");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"zfilename\" ");
  fprintf(cgiOut, "value=%s>", zfilename);
  fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\">");
  fprintf(cgiOut, "<input type=\"submit\" value=\"Show This File\">");
  fprintf(cgiOut, "</td>");
  fprintf(cgiOut, "</form>");
  fprintf(cgiOut, "<form action=\"zenginstore.cgi\" method=\"post\">");
  fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\">");
  fprintf(cgiOut, "<input type=\"submit\" value=\"Show All Files\">");
  fprintf(cgiOut, "</td>");
  fprintf(cgiOut, "</form>");
  fprintf(cgiOut, "</tr>");
  fprintf(cgiOut, "</table>\n");

  pagefoot();
  return(0);
}
